function [ result_depth ] = smoothAdvance( depth, calib, graphpos, upscalefactor,param )

if param.isfillhole
    relativezero=0;
else
    relativezero=1e-3;
end

lambda=[10,1000,10000];
[row,col]=size(depth);
row_scale=upscalefactor;
col_scale=upscalefactor;
grid_scale=[2*min(row_scale,col_scale),2*min(row_scale,col_scale)];
[kx_index,ky_index]=find(calib>relativezero);
kk_index=[kx_index,ky_index];
matrixKindex=zeros(size(calib));
matrixKcalib=zeros(numel(kx_index),1);
matrixKdepth=zeros(numel(kx_index),1);
matrixMindex=zeros(size(calib));
mx_index=zeros(numel(kx_index),1);
my_index=zeros(numel(ky_index),1);
mm_index=zeros(numel(kx_index),2);
for i=1:1:numel(kx_index)
    matrixKindex(kx_index(i),ky_index(i))=i;
    matrixKcalib(i,1)=calib(kx_index(i),ky_index(i));
    matrixKdepth(i,1)=depth(kx_index(i),ky_index(i));
    mx_index(i)=graphpos(kx_index(i),ky_index(i),1);
    my_index(i)=graphpos(kx_index(i),ky_index(i),2);
    mm_index(i,1)=mx_index(i);
    mm_index(i,2)=my_index(i);
    matrixMindex(mx_index(i),my_index(i))=i;
end

matrixKM=zeros(numel(kx_index),numel(mx_index));
for i=1:1:numel(kx_index)
    indexpatch=matrixMindex(max(kx_index(i)-grid_scale(1),1):min(kx_index(i)+grid_scale(1),row),max(ky_index(i)-grid_scale(2),1):min(ky_index(i)+grid_scale(2),col));
    indexpatchcenter=[kx_index(i)+1-max(kx_index(i)-grid_scale(1),1),ky_index(i)+1-max(ky_index(i)-grid_scale(2),1)];
    [i_index,j_index]=find(indexpatch>0);
    weight=(1-sqrt((i_index-indexpatchcenter(1)).^2+(j_index-indexpatchcenter(2)).^2)/sqrt(grid_scale(1)^2+grid_scale(2)^2)).^3;
    for j=1:1:numel(i_index)
        matrixKM(i,indexpatch(i_index(j),j_index(j)))=weight(j,1);
    end
end
matrixKMnorm=sum(matrixKM,2);
matrixKMnorm=max(matrixKMnorm,1e-10);
for i=1:1:size(matrixKM,2)
    matrixKM(:,i)=matrixKM(:,i)./matrixKMnorm;
end

matrixMM=zeros(numel(mx_index),numel(mx_index));
for i=1:1:numel(mx_index)
    indexpatch=matrixMindex(max(mx_index(i)-ceil(row_scale*1.2),1):min(mx_index(i)+ceil(row_scale*1.2),row),max(my_index(i)-ceil(col_scale*1.2),1):min(my_index(i)+ceil(col_scale*1.2),col));
    [i_index,j_index]=find(indexpatch>0);
    for j=1:1:numel(i_index)
        matrixMM(i,indexpatch(i_index(j),j_index(j)))=1;
    end
end

x0_opt = ones(numel(mx_index)*3,1);
A_opt = -speye(numel(mx_index)*3);
for i=1:1:numel(mx_index)
    A_opt(i,i)=0;
    A_opt(numel(mx_index)*2+i,numel(mx_index)*2+i)=0;
end
b_opt = zeros(numel(mx_index)*3,1);
options = optimset('Algorithm','interior-point','Hessian','lbfgs','GradObj', 'on','Display','off','MaxIter',100); % 'MaxIter',100
[x_opt,fval_opt] = fmincon(@gradinline,x0_opt,A_opt,b_opt,[],[],[],[],[],options);

x_opt=reshape(x_opt,numel(mx_index),3);
finalKdepth=zeros(numel(kx_index),1);
for i=1:1:numel(kx_index)
    for j=1:1:numel(mx_index)
        if matrixKM(i,j)>0
            finalKdepth(i)=finalKdepth(i)+matrixKM(i,j)*fval([mx_index(j),my_index(j)],[kx_index(i),ky_index(i)],x_opt(j,:));
        end
    end
    finalKdepth(i)=finalKdepth(i)+matrixKdepth(i,1);
end

[kx_index_all,ky_index_all]=find(depth>0);
matrixKdepth_all=zeros(numel(kx_index_all),1);
for i=1:1:numel(kx_index_all)
    matrixKdepth_all(i,1)=depth(kx_index_all(i),ky_index_all(i));
end
matrixKindex_all=zeros(size(depth));
for i=1:1:numel(kx_index_all)
    matrixKindex_all(kx_index_all(i),ky_index_all(i))=i;
end

matrixKM_all=zeros(numel(kx_index_all),numel(mx_index));
for i=1:1:numel(mx_index)
    indexpatch=matrixKindex_all(max(mx_index(i)-grid_scale(1),1):min(mx_index(i)+grid_scale(1),row),max(my_index(i)-grid_scale(2),1):min(my_index(i)+grid_scale(2),col));
    indexpatchcenter=[mx_index(i)+1-max(mx_index(i)-grid_scale(1),1),my_index(i)+1-max(my_index(i)-grid_scale(2),1)];
    [i_index,j_index]=find(indexpatch>0);
    weight=(1-sqrt((i_index-indexpatchcenter(1)).^2+(j_index-indexpatchcenter(2)).^2)/sqrt(grid_scale(1)^2+grid_scale(2)^2)).^3;
    for j=1:1:numel(i_index)
        matrixKM_all(indexpatch(i_index(j),j_index(j)),i)=weight(j,1);
    end
end

if param.isexpanddetail
    grid_scale=[5*min(row_scale,col_scale),5*min(row_scale,col_scale)];
    matrixKMnorm_all=sum(matrixKM_all,2);
    for i=1:1:numel(mx_index)
        indexpatch=matrixKindex_all(max(mx_index(i)-grid_scale(1),1):min(mx_index(i)+grid_scale(1),row),max(my_index(i)-grid_scale(2),1):min(my_index(i)+grid_scale(2),col));
        indexpatchcenter=[mx_index(i)+1-max(mx_index(i)-grid_scale(1),1),my_index(i)+1-max(my_index(i)-grid_scale(2),1)];
        [i_index,j_index]=find(indexpatch>0);
        weight=(1-sqrt((i_index-indexpatchcenter(1)).^2+(j_index-indexpatchcenter(2)).^2)/sqrt(grid_scale(1)^2+grid_scale(2)^2)).^3;
        for j=1:1:numel(i_index)
            if matrixKMnorm_all(indexpatch(i_index(j),j_index(j)))==0
                matrixKM_all(indexpatch(i_index(j),j_index(j)),i)=weight(j,1);
            end
        end
    end
end

matrixKMnorm_all=sum(matrixKM_all,2);
matrixKMnorm_all=max(matrixKMnorm_all,1e-10);
for i=1:1:size(matrixKM_all,2)
    matrixKM_all(:,i)=matrixKM_all(:,i)./matrixKMnorm_all;
end

result_depth=depth;
for i=1:1:numel(kx_index_all)
    for j=1:1:numel(mx_index)
        if matrixKM_all(i,j)>0
            result_depth(kx_index_all(i),ky_index_all(i))=result_depth(kx_index_all(i),ky_index_all(i))+matrixKM_all(i,j)*fval([mx_index(j),my_index(j)],[kx_index_all(i),ky_index_all(i)],x_opt(j,:));
        end
    end
    result_depth(kx_index_all(i),ky_index_all(i))=result_depth(kx_index_all(i),ky_index_all(i));
end
finalKdepth=result_depth(calib>relativezero);

    function [f_grad, g_grad] = gradinline(x_grad)
        x_grad=reshape(x_grad,numel(mx_index),3);
        f_grad=0;
        [i_index_grad,j_index_grad]=find(matrixMM>0);
        for i_grad=1:1:i_index_grad
            f_grad=f_grad+lambda(1)*(fval(mm_index(j_index_grad(i_grad),:),mm_index(i_index_grad(i_grad),:),x_grad(j_index_grad(i_grad),:)) ...
            -fval(mm_index(j_index_grad(i_grad),:),mm_index(j_index_grad(i_grad),:),x_grad(j_index_grad(i_grad),:)))^2;
        end
        temp_grad=zeros(numel(kx_index),1);
        for i_grad=1:1:numel(kx_index)
            j_grad=find(matrixKM(i_grad,:)>0);
            temp_grad(i_grad)=temp_grad(i_grad)+sum(matrixKM(i_grad,j_grad)'.*fval_inv(mm_index(j_grad,:),kk_index(i_grad,:),x_grad(j_grad,:)));
            temp_grad(i_grad)=temp_grad(i_grad)+matrixKdepth(i_grad,1)-matrixKcalib(i_grad,1);
            f_grad=f_grad+lambda(2)*temp_grad(i_grad)^2;
        end
        for i_grad=1:1:numel(mx_index)
            f_grad=f_grad+lambda(3)*x_grad(i_grad,2);
        end
        g_grad=zeros(numel(mx_index),3);
        for i_grad=1:1:numel(mx_index)
            fval_i_grad=fval(mm_index(i_grad,:),mm_index(i_grad,:),x_grad(i_grad,:));
            fval_theta_i_grad=fval_theta(mm_index(i_grad,:),mm_index(i_grad,:),x_grad(i_grad,:));
            fval_c_i_grad=fval_c(mm_index(i_grad,:),mm_index(i_grad,:),x_grad(i_grad,:));
            j_grad=find(matrixMM(i_grad,:)>0);
            fval_j_i_j=fval_inv(mm_index(j_grad,:),mm_index(i_grad,:),x_grad(j_grad,:));
            fval_i_j_i=fval(mm_index(i_grad,:),mm_index(j_grad,:),x_grad(i_grad,:));
            fval_j_j_j=fval_all(mm_index(j_grad,:),mm_index(j_grad,:),x_grad(j_grad,:));
            g_grad(i_grad,1)=g_grad(i_grad,1)+sum(lambda(1)*(-2*(fval_j_i_j-fval_i_grad)*fval_theta_i_grad+2*(fval_i_j_i-fval_j_j_j).*fval_theta(mm_index(i_grad,:),mm_index(j_grad,:),x_grad(i_grad,:))));
            g_grad(i_grad,2)=g_grad(i_grad,2)+sum(lambda(1)*(-2*(fval_j_i_j-fval_i_grad)*fval_c_i_grad+2*(fval_i_j_i-fval_j_j_j).*fval_c(mm_index(i_grad,:),mm_index(j_grad,:),x_grad(i_grad,:))));
            g_grad(i_grad,3)=g_grad(i_grad,3)+sum(lambda(1)*(-2*(fval_j_i_j-fval_i_grad)+2*(fval_i_j_i-fval_j_j_j)));
            j_grad=find(matrixKM(:,i_grad)>0);
            g_grad(i_grad,1)=g_grad(i_grad,1)+sum(lambda(2)*2*temp_grad(j_grad).*matrixKM(j_grad,i_grad).*fval_theta(mm_index(i_grad,:),kk_index(j_grad,:),x_grad(i_grad,:)));
            g_grad(i_grad,2)=g_grad(i_grad,2)+sum(lambda(2)*2*temp_grad(j_grad).*matrixKM(j_grad,i_grad).*fval_c(mm_index(i_grad,:),kk_index(j_grad,:),x_grad(i_grad,:)));
            g_grad(i_grad,3)=g_grad(i_grad,3)+sum(lambda(2)*2*temp_grad(j_grad).*matrixKM(j_grad,i_grad));
            g_grad(i_grad,2)=g_grad(i_grad,2)+lambda(3);
        end
        g_grad=reshape(g_grad,[numel(mx_index)*3,1]);
    end

    function [value]=fval(gpos,xpos,param)
        theta=param(1);
        c=param(2);
        h=param(3);
        value=((xpos(:,1)-gpos(1))*cos(theta)+(xpos(:,2)-gpos(2))*sin(theta))*c+h;
    end
    function [value]=fval_theta(gpos,xpos,param)
        theta=param(1);
        c=param(2);
        value=(-sin(theta)*(xpos(:,1)-gpos(1))+(xpos(:,2)-gpos(2))*cos(theta))*c;
    end
    function [value]=fval_c(gpos,xpos,param)
        theta=param(1);
        value=((xpos(:,1)-gpos(1))*cos(theta)+(xpos(:,2)-gpos(2))*sin(theta));
    end
    function [value]=fval_h(gpos,xpos,param)
        value=ones(numel(xpos)/2,1);
    end
    function [value]=fval_inv(gpos,xpos,param)
        theta=param(:,1);
        c=param(:,2);
        h=param(:,3);
        value=((xpos(1)-gpos(:,1)).*cos(theta)+(xpos(2)-gpos(:,2)).*sin(theta)).*c+h;
    end
    function [value]=fval_theta_inv(gpos,xpos,param)
        theta=param(:,1);
        c=param(:,2);
        value=(-sin(theta).*(xpos(1)-gpos(:,1))+(xpos(2)-gpos(:,2)).*cos(theta)).*c;
    end
    function [value]=fval_c_inv(gpos,xpos,param)
        theta=param(:,1);
        value=((xpos(1)-gpos(:,1)).*cos(theta)+(xpos(2)-gpos(:,2)).*sin(theta));
    end
    function [value]=fval_h_inv(gpos,xpos,param)
        value=ones(numel(gpos)/2,1);
    end
    function [value]=fval_all(gpos,xpos,param)
        theta=param(:,1);
        c=param(:,2);
        h=param(:,3);
        value=((xpos(:,1)-gpos(:,1)).*cos(theta)+(xpos(:,2)-gpos(:,2)).*sin(theta)).*c+h;
    end

end
