function [ sr_result ] = superResolution( lowdepthdata,seg_image,param,upscalefactor )

sampled_depth=lowdepthdata.sampled_depth;
graphpos1=lowdepthdata.graphpos(:,:,1);
graphpos2=lowdepthdata.graphpos(:,:,2);
seg_index=unique(seg_image);
[row,col]=size(seg_image);
sr_result=zeros(row,col);

if param.isdenoise
    denoiseparam=[0.2,20];
    for seg=1:1:numel(seg_index)
        depthpatch=zeros(size(sampled_depth));
        depthpatch(seg_image==seg)=sampled_depth(seg_image==seg);
        if sum(sum(depthpatch>0))>2
            depthpatch=segDenoise(depthpatch,3*upscalefactor,denoiseparam);
        end
        sampled_depth(seg_image==seg)=depthpatch(seg_image==seg);
    end
end

for seg=1:1:numel(seg_index)
    sampled_depth_segmean=sampled_depth(seg_image==seg);
    sampled_depth_segmean=sampled_depth_segmean(sampled_depth_segmean>0);
    if sampled_depth_segmean>0
        sampled_depth_segmean=mean(sampled_depth_segmean);
    else
        [x_seg,y_seg]=find(seg_image==seg);
        x_seg=mean(x_seg);
        y_seg=mean(y_seg);
        [x_index,y_index]=find(sampled_depth>0);
        Xpos=[x_index,y_index];
        Ypos=[x_seg,y_seg];
        knear=3;
        IDX=knnsearch(Xpos,Ypos,'K',knear);
        sampled_depth_segmean=0;
        for j=1:1:knear
            sampled_depth_segmean=sampled_depth_segmean+sampled_depth(Xpos(IDX(j),1),Xpos(IDX(j),2));
        end
        sampled_depth_segmean=sampled_depth_segmean/knear;
    end
    sr_result(seg_image==seg)=sr_result(seg_image==seg)+sampled_depth_segmean-mean(sr_result(seg_image==seg));
    
    xdata=zeros(size(sampled_depth));
    ydata=zeros(size(sampled_depth));
    gdata=zeros([size(sampled_depth),2]);
    gdata1=zeros(size(sampled_depth));
    gdata2=zeros(size(sampled_depth));
    xdata(seg_image==seg)=sr_result(seg_image==seg);
    ydata(seg_image==seg)=sampled_depth(seg_image==seg);
    gdata1(seg_image==seg)=graphpos1(seg_image==seg);
    gdata2(seg_image==seg)=graphpos2(seg_image==seg);
    gdata(:,:,1)=gdata1;
    gdata(:,:,2)=gdata2;
    if sum(sum(ydata>0))>=4
        if strcmp(param.smoothmethod,'simple')
            result_seg_depth=smoothSimple(xdata,ydata,upscalefactor,param);
        end
        if strcmp(param.smoothmethod,'advance')
            result_seg_depth=smoothAdvance(xdata,ydata,gdata,upscalefactor,param);
        end
        if ~strcmp(param.smoothmethod,'simple') && ~strcmp(param.smoothmethod,'advance')
            disp('Please set smoothmethod method correctly. ');
            exit;
        end
        minval=min(ydata(ydata>0));
        maxval=max(ydata(ydata>0));
        result_seg_depth(seg_image==seg)=min(max(result_seg_depth(seg_image==seg),minval-0.1*(maxval-minval)),maxval+0.1*(maxval-minval));
        sr_result(seg_image==seg)=result_seg_depth(seg_image==seg);
    end
end

end

